import ij.*;
import ij.process.*;
import ij.gui.*;
import ij.io.*;
import ij.plugin.*;
import ij.util.Tools;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

/** This plugin uses the File/Batch/Convert command, 
	which converts the images in a folder to a specified format,
	to generate a dataset with compressed images plus a duplicate copy
	of the original Micro-Manager Metadata.txt file. 

Written 2012/09/25 by Jeff Hardin, Univ. of Wisconsin-Madison
bug fixes 2017/11/14
 */

	public class MicroManager_Compressor implements PlugIn, ActionListener {
		private static final String[] formats = {"TIFF", "8-bit TIFF", "JPEG", "GIF", "PNG", "PGM", "BMP", "FITS", "Text Image", "ZIP", "Raw"};
		private static String format = formats[0];
		//private static int height;
		private static double scale = 1.0;
		private static int interpolationMethod = ImageProcessor.BILINEAR;
		private String[] methods = ImageProcessor.getInterpolationMethods();
		private Button input, output;
		private TextField inputDir, outputDir;
		private GenericDialog gd;

	public void run(String arg) {
        /*DirectoryChooser dd = new DirectoryChooser("Open the dataset directory…");
        if (dd==null) return;
        String dir1 = dd.getDirectory();
        DirectoryChooser dd2 = new DirectoryChooser("Specify the coverted dataset directory…");
        if (dd2==null) return;
        String dir2 = dd2.getDirectory();**/
		if (!showDialog()) return;
        String inputPath = inputDir.getText();
		if (inputPath.equals("")) {
			IJ.error("Batch Converter", "Please choose an input folder");
			return;
		}
        String outputPath = outputDir.getText();
		if (outputPath.equals("")) {
			IJ.error("MicroManager Converter", "Please choose an output folder");
			return;
		}
		File f1 = new File(inputPath);
		if (!f1.exists() || !f1.isDirectory()) {
			IJ.error("MicroManager Converter", "Input does not exist or is not a folder\n \n"+inputPath);
			return;
		}
		File f2 = new File(outputPath);
		if (!outputPath.equals("") && (!f2.exists() || !f2.isDirectory())) {
			IJ.error("MicroManager Converter", "Output does not exist or is not a folder\n \n"+outputPath);
			return;
		}
		String[] list = (new File(inputPath)).list();
		ImageJ ij = IJ.getInstance();
		if (ij!=null) ij.getProgressBar().setBatchMode(true);
		IJ.resetEscape();
		// Get current time
		long start = System.currentTimeMillis();
		for (int i=0; i<list.length; i++) {
			if (IJ.escapePressed()) break;
			if (IJ.debugMode) IJ.log(i+"  "+list[i]);
			//IJ.log(i+"  "+list[i]);
            String path = inputPath + list[i];
			if ((new File(path)).isDirectory()||list[i].endsWith(".txt"))
				continue;
			if (list[i].startsWith(".")||list[i].endsWith(".avi")||list[i].endsWith(".AVI"))
				continue;
			IJ.showProgress(i+1, list.length);
			ImagePlus imp = IJ.openImage(path);
			if (imp==null) continue;
			//if (height!=0) {
			//	double aspectRatio = (double)imp.getWidth()/imp.getHeight();
			//	int width = (int)(height*aspectRatio);
			//	ImageProcessor ip = imp.getProcessor();
			//	ip.setInterpolationMethod(interpolationMethod);
			//	imp.setProcessor(null, ip.resize(width,height));
			//} else 
			if (scale!=1.0) {
				int width = (int)(scale*imp.getWidth());
				int height = (int)(scale*imp.getHeight());
				ImageProcessor ip = imp.getProcessor();
				ip.setInterpolationMethod(interpolationMethod);
				imp.setProcessor(null, ip.resize(width,height,true));
			}
			if (format.equals("8-bit TIFF") || format.equals("GIF")) {
				if (imp.getBitDepth()==24)
					IJ.run(imp, "8-bit Color", "number=256");
				else
					IJ.run(imp, "8-bit", "");
			}
			IJ.saveAs(imp, format, outputPath+list[i]);
			imp.close();
		}
		IJ.showProgress(1,1);
		//Prefs.set("batch.input", inputDir.getText());
		//Prefs.set("batch.output", outputDir.getText());
		copyMetaData(inputPath + "Metadata.txt",outputPath + "Metadata.txt");
		IJ.log("\\Clear");
		IJ.log("Total files processed: " + list.length);
		// Get elapsed time in milliseconds
		long elapsedTimeMillis = System.currentTimeMillis()-start;
		// Get elapsed time in seconds
		float elapsedTimeSec = elapsedTimeMillis/1000F;
		IJ.log("Total processing time (sec): " + elapsedTimeSec);
		//String timeString = getTimeString(elapsedTimeMillis);
		//IJ.log(timeString);
	}
		
	boolean showDialog() {
		gd = new GenericDialog("MicroManager Convert");
		addPanels(gd);
		gd.setInsets(15, 0, 5);
		gd.addChoice("Output Format: ", formats, format);
		gd.addChoice("Interpolation:", methods, methods[interpolationMethod]);
		//gd.addStringField("Height (pixels): ", height==0?"\u2014":""+height, 6);
		gd.addNumericField("Scale Factor: ", scale, 2);
		gd.setOKLabel("Convert");
		gd.showDialog();
		format = gd.getNextChoice();
		interpolationMethod = gd.getNextChoiceIndex();
		//height = (int)Tools.parseDouble(gd.getNextString(), 0.0);
		scale = gd.getNextNumber();
		return !gd.wasCanceled();
	}

	void addPanels(GenericDialog gd) {
		Panel p = new Panel();
    	p.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
		input = new Button("Input...");
		input.addActionListener(this);
		p.add(input);
		inputDir = new TextField(Prefs.get("batch.input", ""), 45);
		//inputDir = new TextField(Prefs.get("", ""), 45);		
		p.add(inputDir);
		gd.addPanel(p);
		p = new Panel();
    	p.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
		output = new Button("Output...");
		output.addActionListener(this);
		p.add(output);
		outputDir = new TextField(Prefs.get("batch.output", ""), 45);
		//outputDir = new TextField(Prefs.get("", ""), 45);
		p.add(outputDir);
		gd.addPanel(p);
	}
	
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		String s = source==input?"Input":"Output";
		String path = IJ.getDirectory(s+" Folder");
		IJ.showMessage(path);
		if (path==null) return;
		if (source==input)
			inputDir.setText(path);
		else
			outputDir.setText(path);
		if (IJ.isMacOSX())
			{gd.setVisible(false); gd.setVisible(true);}
	}

	public void copyMetaData(String origFile, String newFile){
			try{
			File f1 = new File(origFile);
			File f2 = new File(newFile);
			InputStream in = new FileInputStream(f1);
			OutputStream out = new FileOutputStream(f2);
			byte[] buf = new byte[1024];
			int len;
			while ((len = in.read(buf)) > 0){
			out.write(buf, 0 , len);
			}
			in.close();
			out.close();
			//IJ.log("Metadata copied.");
		}
			catch (IOException e)
			{
				IJ.log("There was an error copying the XML file...");
			}
	}

	public String getTimeString (long aux){
	//Calculate the elapsed time in days hours minutes seconds milliseconds;
	// won't show up days, hours, minutes if not needed.
		long days=0, hours=0, minutes = 0, seconds = 0, milliseconds = 0;
		String auxRet = "Time (d/h/m/s/msec):";
		//days
		if (aux > 24*60*60*1000){
			days = aux/(24*60*60*1000);
		}
		aux = aux%(24*60*60*1000);
		//hours
		if (aux > 60*60*1000){
			hours = aux/(60*60*1000);
		}
		aux = aux%(60*60*1000);
		//minutes
		if (aux > 60*1000){
			minutes = aux/(60*1000);
		}
		aux = aux%(60*1000);
			//seconds
		if (aux > 1000){
		seconds = aux/(1000);
		}
		milliseconds = aux%1000;
		if(days>0){
			auxRet = days +" : " ;
		}
		if(days != 0 || hours>0){
			auxRet += hours+ " : " ;
		}
		if(days != 0 || hours!= 0 || minutes>0){
			auxRet += minutes +" : " ;
		}
		if(days != 0 || hours!= 0 || minutes!=0 || seconds>0){
			auxRet += seconds + " . ";
		}
		auxRet +=  milliseconds;
		return auxRet;
	}

}
